@@ -97,7 +97,8 @@ static void DoSetFocus(Window w, FvwmWindow *Fw, Bool FocusByMouse, Bool NoWarp)
    * Focus-by-mouse type focus events */
   /* Watch out: Fw may not be on the windowlist and the windowlist may be
    * empty */
-  if (Fw && Fw != get_focus_window() && Fw != &Scr.FvwmRoot)
+  if (Fw && Fw != get_focus_window() && Fw != &Scr.FvwmRoot &&
+      !IS_SCHEDULED_FOR_DESTROY(Fw))
   {
     if (FocusByMouse) /* pluck window from list and deposit at top */
     {
@@ -348,9 +349,9 @@ void restore_focus_after_unmap(
     {
       for (t = tmp_win->next; t != NULL; t = t->next)
       {
-	if (t->Desk == tmp_win->Desk &&
-	    !DO_SKIP_CIRCULATE(t) &&
-	    !(DO_SKIP_ICON_CIRCULATE(t) && IS_ICONIFIED(t)) &&
+	if (t->Desk == tmp_win->Desk && !DO_SKIP_CIRCULATE(t) &&
+	    !(IS_ICONIFIED(t) && (
+		      DO_SKIP_ICON_CIRCULATE(t) || IS_ICON_SUPPRESSED(t))) &&
 	    (!do_skip_marked_transients || !IS_IN_TRANSIENT_SUBTREE(t)))
 	{
 	  /* If it is on a different desk we have to look for another window */
