@@ -39,6 +39,8 @@
 #endif
 
 #include "libs/fvwmlib.h"
+#include "libs/FScreen.h"
+#include "libs/FShape.h"
 #include "fvwm.h"
 #include "externs.h"
 #include "cursor.h"
@@ -66,17 +68,14 @@
 #include "focus.h"
 #include "update.h"
 #include "window_flags.h"
+#include "move_resize.h"
 
 #include <X11/Xproto.h>
 #include <X11/Xatom.h>
 /* need to get prototype for XrmUniqueQuark for XUniqueContext call */
 #include <X11/Xresource.h>
 
-#ifdef SHAPE
-#include <X11/extensions/shape.h>
-#endif /* SHAPE */
-
-#ifdef I18N_MB
+#if defined(I18N_MB) || defined(COMPOUND_TEXT)
 #include <X11/Xlocale.h>
 #endif
 
@@ -113,7 +112,6 @@ int CatchFatal(Display *);
 int CatchRedirectError(Display *, XErrorEvent *);
 void InstallSignals(void);
 void SaveDesktopState(void);
-void SetMWM_INFO(Window window);
 void SetRCDefaults(void);
 void StartupStuff(void);
 static void catch_exit(void);
@@ -125,7 +123,6 @@ static int parseCommandArgs(
 static void InternUsefulAtoms(void);
 static void InitVariables(void);
 static void usage(void);
-static int MappedNotOverride(Window w);
 static void setVersionInfo(void);
 
 XContext FvwmContext;		/* context for fvwm windows */
@@ -136,7 +133,7 @@ Window JunkRoot, JunkChild;		/* junk window */
 unsigned int JunkWidth, JunkHeight, JunkBW, JunkDepth, JunkMask;
 
 Bool debugging = False;
-Bool PPosOverride = False;
+Bool debugging_stack_ring = False;
 
 Window bad_window = None;
 
@@ -159,13 +156,6 @@ static char l_g_bits[] = {0x08, 0x02};
 #define s_g_height 4
 static char s_g_bits[] = {0x01, 0x02, 0x04, 0x08};
 
-#ifdef SHAPE
-int ShapeEventBase, ShapeErrorBase;
-Boolean ShapesSupported=False;
-#endif
-
-long isIconicState = 0;
-Bool isIconifiedByParent = False;
 extern XEvent Event;
 Bool Restarting = False;
 int fd_width, x_fd;
@@ -200,18 +190,37 @@ int main(int argc, char **argv)
   Bool option_error = False;
   int visualClass = -1;
   int visualId = -1;
-  int x, y;
 
-  g_argv = argv;
+  /* for use on restart */
+  g_argv = (char **)safemalloc((argc + 4) * sizeof(char *));
   g_argc = argc;
+  for (i = 0; i < argc; i++)
+    g_argv[i] = argv[i];
+  g_argv[g_argc] = NULL;
 
   DBUG("main","Entered, about to parse args");
 
   setVersionInfo();
 
+#if defined(I18N_MB) || defined(COMPOUND_TEXT)
+  {
+    char *s;
+
+    if ((s = setlocale(LC_CTYPE, "")) == NULL)
+    {
+      fvwm_msg(ERR, "main","Can't set locale. Check your $LC_CTYPE or $LANG.\n");
+    }
+    else if (!XSupportsLocale())
+    {
+      fvwm_msg(WARN, "main", "X does not support current locale: %s\n", s);
+    }
 #ifdef I18N_MB
-  if (setlocale(LC_CTYPE, "") == NULL)
-    fvwm_msg(ERR, "main", "Can't set locale. Check your $LC_CTYPE or $LANG.\n");
+    else if (XSetLocaleModifiers("") == NULL)
+    {
+      fvwm_msg(WARN, "main", "Cannot set locale modifiers\n");
+    }
+#endif
+  }
 #endif
 
   /* Put the default module directory into the environment so it can be used
@@ -224,20 +233,20 @@ int main(int argc, char **argv)
   if (home_dir == NULL) {
     struct passwd* pw = getpwuid(getuid());
     if (pw != NULL)
-      home_dir = strdup(pw->pw_dir);
+      home_dir = safestrdup(pw->pw_dir);
   }
 #endif
   if (home_dir == NULL)
-    home_dir = "."; /* give up and use current dir */
+    home_dir = "/"; /* give up and use root dir */
 
   /* Figure out where to read and write user's data files. */
   fvwm_userdir = getenv("FVWM_USERDIR");
   if (fvwm_userdir == NULL)
   {
-    fvwm_userdir = strdup(CatString2(home_dir, "/.fvwm"));
+    fvwm_userdir = safestrdup(CatString2(home_dir, "/.fvwm"));
     /* Put the user directory into the environment so it can be used
        later everywhere. */
-    putenv(strdup(CatString2("FVWM_USERDIR=", fvwm_userdir)));
+    putenv(safestrdup(CatString2("FVWM_USERDIR=", fvwm_userdir)));
   }
 
   /* Create FVWM_USERDIR directory if needed */
@@ -248,22 +257,26 @@ int main(int argc, char **argv)
 
   for (i = 1; i < argc; i++)
   {
-    if (strncasecmp(argv[i],"-debug",6)==0)
+    if (strncasecmp(argv[i],"-debug_stack_ring", 17)==0)
+    {
+      debugging_stack_ring = True;
+    }
+    else if (strncasecmp(argv[i],"-debug",6)==0)
     {
       debugging = True;
     }
     else if (strncasecmp(argv[i], "-clientId", 9) == 0)
-      {
-	if (++i >= argc)
-	  usage();
-	SetClientID(argv[i]);
-      }
+    {
+      if (++i >= argc)
+	usage();
+      SetClientID(argv[i]);
+    }
     else if (strncasecmp(argv[i], "-restore", 8) == 0)
-      {
-	if (++i >= argc)
-	  usage();
-	state_filename = argv[i];
-      }
+    {
+      if (++i >= argc)
+	usage();
+      state_filename = argv[i];
+    }
     else if (strncasecmp(argv[i],"-s",2)==0)
     {
       do_force_single_screen = True;
@@ -297,7 +310,7 @@ int main(int argc, char **argv)
         usage();
       if (num_config_commands < MAX_CFG_CMDS)
       {
-        config_commands[num_config_commands] = strdup(argv[i]);
+        config_commands[num_config_commands] = safestrdup(argv[i]);
         num_config_commands++;
       }
       else
@@ -356,7 +369,7 @@ int main(int argc, char **argv)
     }
     else
     {
-      fvwm_msg(ERR,"main","Unknown option:  `%s'\n", argv[i]);
+      fvwm_msg(ERR,"main","Unknown option: `%s'", argv[i]);
       option_error = TRUE;
     }
   }
@@ -414,8 +427,17 @@ int main(int argc, char **argv)
         break;
       }
     }
+
+    g_argv[argc++] = "-s";
+/*
+    g_argv[argc++] = "-d";
+    g_argv[argc++] = safestrdup(XDisplayString(dpy));
+*/
+    g_argv[argc] = NULL;
   }
 
+  FScreenInit(dpy);
+
   x_fd = XConnectionNumber(dpy);
   fd_width = GetFdWidth();
 
@@ -531,9 +553,7 @@ int main(int argc, char **argv)
   /* make a copy of the visual stuff so that XorPixmap can swap with root */
   SaveFvwmVisual();
 
-#ifdef SHAPE
-  ShapesSupported = XShapeQueryExtension(dpy, &ShapeEventBase, &ShapeErrorBase);
-#endif /* SHAPE */
+  FShapeInit(dpy);
   Scr.pscreen = XScreenOfDisplay(dpy, Scr.screen);
   Scr.use_backing_store = DoesBackingStore(Scr.pscreen);
   Scr.flags.do_save_under = DoesSaveUnders(Scr.pscreen);
@@ -548,7 +568,7 @@ int main(int argc, char **argv)
      windows have it */
   /* do this before any RC parsing as some GC's are created from this window
    * rather than the root window */
-  attributes.event_mask = NO_FOCUS_WIN_EVMASK;
+  attributes.event_mask = XEVMASK_NOFOCUSW;
   attributes.override_redirect = True;
   attributes.colormap = Pcmap;
   attributes.background_pixmap = None;
@@ -569,13 +589,7 @@ int main(int argc, char **argv)
   SetupICCCM2 (replace_wm);
   XSetErrorHandler(CatchRedirectError);
   XSetIOErrorHandler(CatchFatal);
-  XSelectInput(dpy, Scr.Root,
-               LeaveWindowMask| EnterWindowMask | PropertyChangeMask |
-               SubstructureRedirectMask | KeyPressMask |
-               SubstructureNotifyMask|
-	       ColormapChangeMask|
-	       STROKE_CODE(ButtonMotionMask | DEFAULT_ALL_BUTTONS_MOTION_MASK |)
-               ButtonPressMask | ButtonReleaseMask );
+  XSelectInput(dpy, Scr.Root, XEVMASK_ROOTW);
   XSync(dpy, 0);
 
   XSetErrorHandler(FvwmErrorHandler);
@@ -608,7 +622,8 @@ int main(int argc, char **argv)
   }
 
   restart_state_filename =
-    strdup(CatString3(fvwm_userdir, "/.fs-restart-", getenv("HOSTDISPLAY")));
+    safestrdup(
+      CatString3(fvwm_userdir, "/.fs-restart-", getenv("HOSTDISPLAY")));
   if (!state_filename && Restarting)
     state_filename = restart_state_filename;
 
@@ -693,32 +708,15 @@ int main(int argc, char **argv)
 				  WhitePixel(dpy, Scr.screen), Pdepth);
   }
 
-  /* create the move/resize feedback window */
-  Scr.SizeStringWidth = XTextWidth (Scr.DefaultFont.font,
-                                    " +8888 x +8888 ", 15);
   attributes.background_pixel = Scr.StdBack;
   attributes.colormap = Pcmap;
   attributes.border_pixel = 0;
   valuemask = CWBackPixel | CWColormap | CWBorderPixel;
 
-  if(!Scr.gs.EmulateMWM)
-  {
-    x = 0;
-    y = 0;
-  }
-  else
-  {
-    x = Scr.MyDisplayWidth/2 - (Scr.SizeStringWidth + SIZE_HINDENT*2)/2;
-    y = Scr.MyDisplayHeight/2 - (Scr.DefaultFont.height + SIZE_VINDENT*2)/2;
-  }
-  Scr.SizeWindow = XCreateWindow (dpy, Scr.Root, x, y,
-				  (unsigned int)(Scr.SizeStringWidth +
-						 SIZE_HINDENT*2),
-				  (unsigned int) (Scr.DefaultFont.height +
-						  SIZE_VINDENT*2),
-				  (unsigned int) 0, Pdepth,
-				  InputOutput, Pvisual,
-				  valuemask, &attributes);
+  Scr.SizeWindow = XCreateWindow(
+    dpy, Scr.Root, 0, 0, 1, 1, (unsigned int)0, Pdepth, InputOutput, Pvisual,
+    valuemask, &attributes);
+  resize_geometry_window();
   initPanFrames();
 
   MyXGrabServer(dpy);
@@ -776,11 +774,11 @@ void StartupStuff(void)
 #define startFuncName "StartFunction"
   const char *initFuncName;
 
-  CaptureAllWindows();
+  CaptureAllWindows(False);
   /* Turn off the SM stuff after the initial capture so that new windows will
    * not be matched by accident. */
   if (Restarting)
-    DisableSM();
+    DisableRestoringState();
   /* Have to do this here too because preprocessor modules have not run to the
    * end when HandleEvents is entered from the main loop. */
   checkPanFrames();
@@ -791,10 +789,16 @@ void StartupStuff(void)
   if (Scr.ClickTime < 0)
     Scr.ClickTime = -Scr.ClickTime;
 
-  /* It is safe to Ungrab here: if not and one of the init functions not
-   * finish we've got a complete freeze ! */
+# if 0
+  /* It is safe to ungrab here: if not, and one of the init functions
+   * does not finish, we've got a complete freeze! */
+  /* DV (15-Jul-2004): No, it is not safe to ungrab.  If another application
+   * grabs the pointer before execute_function gets it, the start functions
+   * are not executed.  And the pointer is grabbed during function execution
+   * anyway, so releasing it here buys us nothing. */
   UngrabEm(GRAB_STARTUP);
   XUngrabPointer(dpy, CurrentTime);
+#endif
 
   /* migo (04-Sep-1999): execute StartFunction */
   if (FindFunction(startFuncName)) {
@@ -805,10 +809,13 @@ void StartupStuff(void)
   /* migo (03-Jul-1999): execute [Session]{Init|Restart}Function */
   initFuncName = getInitFunctionName(Restarting == True);
   if (FindFunction(initFuncName)) {
-    char *action = strdup(CatString2("Function ", initFuncName));
+    char *action = safestrdup(CatString2("Function ", initFuncName));
     old_execute_function(action, NULL, &Event, C_ROOT, 1, 0, NULL);
     free(action);
   }
+  /* see comment above */
+  UngrabEm(GRAB_STARTUP);
+  XUngrabPointer(dpy, CurrentTime);
 
   /*
      This should be done after the initialization is finished, since
@@ -826,245 +833,6 @@ void StartupStuff(void)
 } /* StartupStuff */
 
 
-/***********************************************************************
- *
- *  Procedure:
- *      CaptureOneWindow
- *      CaptureAllWindows
- *
- *   Decorates windows at start-up and during recaptures
- *
- ***********************************************************************/
-
-void CaptureOneWindow(
-  FvwmWindow *fw, Window window, Window keep_on_top_win, Window parent_win)
-{
-  Window w;
-  unsigned long data[1];
-
-  isIconicState = DontCareState;
-  if (fw == NULL)
-  {
-    return;
-  }
-  if (IS_SCHEDULED_FOR_DESTROY(fw))
-  {
-    /* Fvwm might crash in complex functions if we really try to the dying
-     * window here because AddWindow() may fail and leave a destroyed window
-     * in Tmp_win.  Any, by the way, it is pretty useless to recapture a
-     * window that will vanish in a moment. */
-    return;
-  }
-  /* Grab the server to make sure the window does not die during the recapture.
-   */
-  MyXGrabServer(dpy);
-  if (!XGetGeometry(dpy, fw->w, &JunkRoot, &JunkX, &JunkY, &JunkWidth,
-		   &JunkHeight, &JunkBW,  &JunkDepth))
-  {
-    /* The window has already died, do not recapture it! */
-    MyXUngrabServer(dpy);
-    return;
-  }
-  if (XFindContext(dpy, window, FvwmContext, (caddr_t *)&fw) != XCNOENT)
-  {
-    Bool f = PPosOverride;
-    Bool is_mapped = IS_MAPPED(fw);
-
-    PPosOverride = True;
-    if (IS_ICONIFIED(fw))
-    {
-      isIconicState = IconicState;
-      isIconifiedByParent = IS_ICONIFIED_BY_PARENT(fw);
-    }
-    else
-    {
-      isIconicState = NormalState;
-      isIconifiedByParent = 0;
-      if (Scr.CurrentDesk != fw->Desk)
-	SetMapStateProp(fw, NormalState);
-    }
-    data[0] = (unsigned long) fw->Desk;
-    XChangeProperty (dpy, fw->w, _XA_WM_DESKTOP, _XA_WM_DESKTOP, 32,
-                     PropModeReplace, (unsigned char *) data, 1);
-
-    GNOME_SetHints(fw);
-    GNOME_SetDesk(fw);
-    GNOME_SetLayer(fw);
-    GNOME_SetWinArea(fw);
-
-    XSelectInput(dpy, fw->w, 0);
-    w = fw->w;
-    XUnmapWindow(dpy, fw->frame);
-    RestoreWithdrawnLocation(fw, True, parent_win);
-    SET_DO_REUSE_DESTROYED(fw, 1); /* RBW - 1999/03/20 */
-    destroy_window(fw);
-    Event.xmaprequest.window = w;
-    HandleMapRequestKeepRaised(keep_on_top_win, fw);
-    if (!fFvwmInStartup)
-    {
-      SET_MAP_PENDING(fw, 0);
-      SET_MAPPED(fw, is_mapped);
-    }
-    /* Clean out isIconicState here, otherwise all new windos may start
-     * iconified. */
-    isIconicState = DontCareState;
-    /* restore previous value */
-    PPosOverride = f;
-  }
-  MyXUngrabServer(dpy);
-
-  return;
-}
-
-/* Put a transparent window all over the screen to hide what happens below. */
-static void hide_screen(
-  Bool do_hide, Window *ret_hide_win, Window *ret_parent_win)
-{
-  static Bool is_hidden = False;
-  static Window hide_win = None;
-  static Window parent_win = None;
-  XSetWindowAttributes xswa;
-  unsigned long valuemask;
-
-  if (do_hide == is_hidden)
-  {
-    /* nothing to do */
-    if (ret_hide_win)
-      *ret_hide_win = hide_win;
-    if (ret_parent_win)
-      *ret_parent_win = parent_win;
-
-    return;
-  }
-  is_hidden = do_hide;
-  if (do_hide)
-  {
-    xswa.override_redirect = True;
-    xswa.cursor = Scr.FvwmCursors[CRS_WAIT];
-    valuemask = CWOverrideRedirect|CWCursor;
-    hide_win = XCreateWindow(
-      dpy, Scr.Root, 0, 0, Scr.MyDisplayWidth, Scr.MyDisplayHeight, 0,
-      CopyFromParent, InputOutput, CopyFromParent, valuemask, &xswa);
-    if (hide_win)
-    {
-      /* When recapturing, all windows are reparented to this window. If they
-       * are reparented to the root window, they will flash over the hide_win
-       * with XFree.  So reparent them to an unmapped window that looks like
-       * the root window. */
-      parent_win = XCreateWindow(
-	dpy, Scr.Root, 0, 0, Scr.MyDisplayWidth, Scr.MyDisplayHeight, 0,
-	CopyFromParent, InputOutput, CopyFromParent, valuemask, &xswa);
-      if (!parent_win)
-      {
-	XDestroyWindow(dpy, hide_win);
-	hide_win = None;
-      }
-      else
-      {
-	XMapWindow(dpy, hide_win);
-	XSync(dpy, 0);
-      }
-    }
-  }
-  else
-  {
-    XDestroyWindow(dpy, hide_win);
-    XDestroyWindow(dpy, parent_win);
-    XSync(dpy,0);
-    hide_win = None;
-  }
-  if (ret_hide_win)
-    *ret_hide_win = hide_win;
-  if (ret_parent_win)
-    *ret_parent_win = parent_win;
-
-  return;
-}
-
-void CaptureAllWindows(void)
-{
-  int i,j;
-  unsigned int nchildren;
-  Window root, parent, *children;
-  FvwmWindow *tmp;		/* temp fvwm window structure */
-
-  MyXGrabServer(dpy);
-
-  if(!XQueryTree(dpy, Scr.Root, &root, &parent, &children, &nchildren))
-  {
-    MyXUngrabServer(dpy);
-    return;
-  }
-
-  PPosOverride = True;
-  if (!(Scr.flags.windows_captured)) /* initial capture? */
-  {
-    /*
-    ** weed out icon windows
-    */
-    for (i=0;i<nchildren;i++)
-    {
-      if (children[i])
-      {
-        XWMHints *wmhintsp = XGetWMHints (dpy, children[i]);
-        if (wmhintsp)
-        {
-          if (wmhintsp->flags & IconWindowHint)
-          {
-            for (j = 0; j < nchildren; j++)
-            {
-              if (children[j] == wmhintsp->icon_window)
-              {
-                children[j] = None;
-                break;
-              }
-            }
-          }
-          XFree ((char *) wmhintsp);
-        }
-      }
-    }
-    /*
-    ** map all of the non-override, non-icon windows
-    */
-    for (i=0;i<nchildren;i++)
-    {
-      if (children[i] && MappedNotOverride(children[i]))
-      {
-        XUnmapWindow(dpy, children[i]);
-        Event.xmaprequest.window = children[i];
-        HandleMapRequestKeepRaised (None, NULL);
-      }
-    }
-    Scr.flags.windows_captured = 1;
-  }
-  else /* must be recapture */
-  {
-    Window keep_on_top_win;
-    Window parent_win;
-
-    hide_screen(True, &keep_on_top_win, &parent_win);
-    /* reborder all windows */
-    for (i=0;i<nchildren;i++)
-    {
-      if (XFindContext(dpy, children[i], FvwmContext, (caddr_t *)&tmp)!=XCNOENT)
-      {
-        CaptureOneWindow(tmp, children[i], keep_on_top_win, parent_win);
-      }
-    }
-    hide_screen(False, NULL, NULL);
-  }
-
-  isIconicState = DontCareState;
-
-  if(nchildren > 0)
-    XFree((char *)children);
-
-  /* after the windows already on the screen are in place,
-   * don't use PPosition */
-  PPosOverride = False;
-  MyXUngrabServer(dpy);
-}
 
 /*
 ** SetRCDefaults
@@ -1078,7 +846,7 @@ void SetRCDefaults(void)
     "XORValue 0",
     "DefaultFont fixed",
     "DefaultColors black grey",
-    "MenuStyle * fvwm, Foreground black, Background grey, Greyed slategrey",
+    DEFAULT_MENU_STYLE,
     "TitleStyle Centered -- Raised",
     "Style * Color lightgrey/dimgrey",
     "Style * HilightFore black, HilightBack grey",
@@ -1102,51 +870,6 @@ void SetRCDefaults(void)
   }
 } /* SetRCDefaults */
 
-/***********************************************************************
- *
- *  Procedure:
- *	MappedNotOverride - checks to see if we should really
- *		put a fvwm frame on the window
- *
- *  Returned Value:
- *	TRUE	- go ahead and frame the window
- *	FALSE	- don't frame the window
- *
- *  Inputs:
- *	w	- the window to check
- *
- ***********************************************************************/
-
-static int MappedNotOverride(Window w)
-{
-  XWindowAttributes wa;
-  Atom atype;
-  int aformat;
-  unsigned long nitems, bytes_remain;
-  unsigned char *prop;
-
-  isIconicState = DontCareState;
-
-  if((w==Scr.NoFocusWin)||(!XGetWindowAttributes(dpy, w, &wa)))
-    return False;
-
-  if(XGetWindowProperty(dpy,w,_XA_WM_STATE,0L,3L,False,_XA_WM_STATE,
-			&atype,&aformat,&nitems,&bytes_remain,&prop)==Success)
-  {
-    if(prop != NULL)
-    {
-      isIconicState = *(long *)prop;
-      XFree(prop);
-    }
-  }
-  if(wa.override_redirect == True)
-  {
-    XSelectInput(dpy,w,FocusChangeMask);
-  }
-  return (((isIconicState == IconicState)||(wa.map_state != IsUnmapped)) &&
-	  (wa.override_redirect != True));
-}
-
 
 /***********************************************************************
  *
@@ -1597,7 +1320,7 @@ void DestroyAllButtons(FvwmDecor *decor)
 
   for (tbp = decor->buttons, i = 0; i < NUMBER_OF_BUTTONS; i++, tbp++)
   {
-    for (j = 0, face = TB_STATE(*tbp); j < MaxButtonState; j++, face++)
+    for (j = 0, face = TB_STATE(*tbp); j < BS_MaxButtonState; j++, face++)
     {
       FreeDecorFace(dpy, face);
     }
@@ -1617,7 +1340,7 @@ void ResetAllButtons(FvwmDecor *decor)
   {
     memset(&TB_FLAGS(*tbp), 0, sizeof(TB_FLAGS(*tbp)));
     TB_JUSTIFICATION(*tbp) = JUST_CENTER;
-    for (face = TB_STATE(*tbp), j = 0; j < MaxButtonState; j++, face++)
+    for (face = TB_STATE(*tbp), j = 0; j < BS_MaxButtonState; j++, face++)
     {
       LoadDefaultButton(face, i);
     }
@@ -1650,9 +1373,11 @@ static void CreateGCs(void)
 
   Scr.ScratchGC1 = fvwmlib_XCreateGC(dpy, Scr.NoFocusWin, gcm, &gcv);
   Scr.ScratchGC2 = fvwmlib_XCreateGC(dpy, Scr.NoFocusWin, gcm, &gcv);
+  Scr.ScratchGC3 = fvwmlib_XCreateGC(dpy, Scr.NoFocusWin, gcm, &gcv);
+  Scr.ScratchGC4 = fvwmlib_XCreateGC(dpy, Scr.NoFocusWin, gcm, &gcv);
   Scr.TitleGC = fvwmlib_XCreateGC(dpy, Scr.NoFocusWin, gcm, &gcv);
   Scr.TransMaskGC = fvwmlib_XCreateGC(dpy, Scr.NoFocusWin, gcm, &gcv);
-  c.pixel = GetColor("Black");
+  c.pixel = GetColor(DEFAULT_FORE_COLOR);
   Scr.ScratchMonoPixmap = XCreatePixmap(dpy, Scr.Root, 1, 1, 1);
   Scr.MonoGC = fvwmlib_XCreateGC(dpy, Scr.ScratchMonoPixmap, gcm, &gcv);
 }
@@ -1684,8 +1409,8 @@ static void InitVariables(void)
   AllocColorset(0);
   /* set up colorset 0 so that if FvwmTheme fails to start any modules
    * using colorsets don't appear black on black */
-  Colorset[0].fg = Scr.StdFore = GetColor("black");
-  Colorset[0].bg = Scr.StdBack = GetColor("gray");
+  Colorset[0].fg = Scr.StdFore = GetColor(DEFAULT_FORE_COLOR);
+  Colorset[0].bg = Scr.StdBack = GetColor(DEFAULT_BACK_COLOR);
   Colorset[0].hilite = Scr.StdHilite = GetHilite(Scr.StdBack);
   Colorset[0].shadow = Scr.StdShadow = GetShadow(Scr.StdBack);
   Scr.StdGC = 0;
@@ -1723,11 +1448,8 @@ static void InitVariables(void)
   Scr.MyDisplayHeight = DisplayHeight(dpy, Scr.screen);
   Scr.BusyCursor = BUSY_NONE;
   Scr.Hilite = NULL;
-  Scr.Focus = NULL;
-  Scr.PreviousFocus = NULL;
   /* this indicates that the root window was never entered since the startup of
    * fvwm. */
-  Scr.LastScreenFocus = &Scr.FvwmRoot;
   Scr.Ungrabbed = NULL;
 
   Scr.DefaultFont.font = NULL;
@@ -1744,9 +1466,11 @@ static void InitVariables(void)
    * compilations */
   Scr.CurrentDesk = 0;
 
-  Scr.EdgeScrollX = Scr.EdgeScrollY = DEFAULT_EDGE_SCROLL;
+  Scr.EdgeScrollX = DEFAULT_EDGE_SCROLL * Scr.MyDisplayWidth / 100;
+  Scr.EdgeScrollY = DEFAULT_EDGE_SCROLL * Scr.MyDisplayHeight / 100;
   Scr.ScrollResistance = DEFAULT_SCROLL_RESISTANCE;
   Scr.MoveResistance = DEFAULT_MOVE_RESISTANCE;
+  Scr.XiMoveResistance = DEFAULT_XIMOVE_RESISTANCE;
   Scr.SnapAttraction = DEFAULT_SNAP_ATTRACTION;
   Scr.SnapMode = DEFAULT_SNAP_ATTRACTION_MODE;
   Scr.SnapGridX = DEFAULT_SNAP_GRID_X;
@@ -1778,69 +1502,32 @@ static void InitVariables(void)
 
      "Network Computing Devices Inc."
         is the ServerVendor string of the PCXware X server under Windows.
+
+     "WRQ, Inc."
+        is the ServerVendor string of the Reflection X server under Windows.
   */
   Scr.bo.RaiseHackNeeded =
     (strcmp (ServerVendor (dpy), "Hummingbird Communications Ltd.") == 0) ||
-    (strcmp (ServerVendor (dpy), "Network Computing Devices Inc.") == 0);
+    (strcmp (ServerVendor (dpy), "Network Computing Devices Inc.") == 0) ||
+    (strcmp (ServerVendor (dpy), "WRQ, Inc.") == 0);
 
-#ifdef MODALITY_IS_EVIL
-  Scr.bo.ModalityIsEvil = 1;
-#else
   Scr.bo.ModalityIsEvil = 0;
-#endif
-#ifdef DISABLE_CONFIGURE_NOTIFY_DURING_MOVE
-  Scr.bo.DisableConfigureNotify = 1;
-#else
   Scr.bo.DisableConfigureNotify = 0;
-#endif
-
   Scr.bo.InstallRootCmap = 0;
+  Scr.bo.FlickeringQtDialogsWorkaround = 1;
 
   Scr.gs.EmulateMWM = DEFAULT_EMULATE_MWM;
   Scr.gs.EmulateWIN = DEFAULT_EMULATE_WIN;
   Scr.gs.use_active_down_buttons = DEFAULT_USE_ACTIVE_DOWN_BUTTONS;
   Scr.gs.use_inactive_buttons = DEFAULT_USE_INACTIVE_BUTTONS;
   /* Not the right place for this, should only be called once somewhere .. */
+  Scr.flags.is_pointer_on_this_screen = !!XQueryPointer(
+	  dpy, Scr.Root, &JunkRoot, &JunkChild, &JunkX, &JunkY, &JunkX, &JunkY,
+	  &JunkMask);
 
   return;
 }
 
-/***********************************************************************
- *
- *  Procedure:
- *	Reborder - Removes fvwm border windows
- *
- ************************************************************************/
-static void Reborder(void)
-{
-  FvwmWindow *tmp;			/* temp fvwm window structure */
-
-  /* put a border back around all windows */
-  MyXGrabServer (dpy);
-
-  InstallWindowColormaps (&Scr.FvwmRoot);	/* force reinstall */
-/*
-    RBW - 05/15/1998
-    Grab the last window and work backwards: preserve stacking order on restart.
-*/
-  for (tmp = Scr.FvwmRoot.stack_prev; tmp != &Scr.FvwmRoot;
-       tmp = tmp->stack_prev)
-  {
-    if (!IS_ICONIFIED(tmp) && Scr.CurrentDesk != tmp->Desk)
-    {
-      XMapWindow(dpy, tmp->w);
-      SetMapStateProp(tmp, NormalState);
-    }
-    RestoreWithdrawnLocation (tmp, True, Scr.Root);
-    XUnmapWindow(dpy,tmp->frame);
-    XDestroyWindow(dpy,tmp->frame);
-  }
-
-  MyXUngrabServer (dpy);
-  FOCUS_RESET();
-  XSync(dpy,0);
-}
-
 /***********************************************************************
  *
  *  Procedure:
@@ -1880,7 +1567,7 @@ void Done(int restart, char *command)
   exitFuncName = getInitFunctionName(2);
   if (FindFunction(exitFuncName))
   {
-    char *action = strdup(CatString2("Function ", exitFuncName));
+    char *action = safestrdup(CatString2("Function ", exitFuncName));
     old_execute_function(action, NULL, &Event, C_ROOT, 1, 0, NULL);
     free(action);
   }
@@ -1933,8 +1620,6 @@ void Done(int restart, char *command)
     if (command)
     {
 #define MAX_ARG_SIZE 25
-      /* This is not allowed by ANSI C! Must use a macro. */
-      /* const int MAX_ARG_SIZE = 25; */
       char *my_argv[MAX_ARG_SIZE];
       const char *errorMsg;
       int n = parseCommandArgs(command, my_argv, MAX_ARG_SIZE, &errorMsg);
@@ -1972,6 +1657,14 @@ void Done(int restart, char *command)
       }
       else
       {
+        /* Warn against an old 'Restart fvwm2' usage */
+        if (n == 1 && strcmp(my_argv[0], "fvwm2") == 0)
+        {
+          fvwm_msg(WARN, "Done",
+            "`Restart fvwm2' might not do what you want, see the man page.\n\t"
+            "Use Restart without parameters if you mean to restart the same WM."
+          );
+        }
         execvp(my_argv[0], my_argv);
         fvwm_msg(ERR, "Done", "Call of '%s' failed! (restarting '%s' instead)",
           my_argv[0], g_argv[0]);
@@ -2006,6 +1699,8 @@ int CatchRedirectError(Display *dpy, XErrorEvent *event)
 {
   fvwm_msg(ERR,"CatchRedirectError","another WM is running");
   exit(1);
+  /* to make insure happy */
+  return 0;
 }
 
 /***********************************************************************
@@ -2021,6 +1716,8 @@ int CatchFatal(Display *dpy)
      */
   ClosePipes();
   exit(1);
+  /* to make insure happy */
+  return 0;
 }
 
 /***********************************************************************
@@ -2040,13 +1737,16 @@ int FvwmErrorHandler(Display *dpy, XErrorEvent *event)
   }
   /* some errors are acceptable, mostly they're caused by
    * trying to update a lost  window or free'ing another modules colors */
-  if((event->error_code == BadWindow)||(event->request_code == X_GetGeometry)||
-     (event->error_code==BadDrawable)||(event->request_code==X_SetInputFocus)||
-     (event->request_code==X_GrabButton)||
-     (event->request_code==X_ChangeWindowAttributes)||
-     (event->request_code == X_InstallColormap) ||
-     (event->request_code == X_FreePixmap) ||
-     (event->request_code == X_FreeColors))
+  if(event->error_code == BadWindow ||
+     event->request_code == X_GetGeometry ||
+     event->error_code == BadDrawable ||
+     event->request_code == X_ConfigureWindow ||
+     event->request_code == X_SetInputFocus||
+     event->request_code == X_GrabButton ||
+     event->request_code == X_ChangeWindowAttributes ||
+     event->request_code == X_InstallColormap ||
+     event->request_code == X_FreePixmap ||
+     event->request_code == X_FreeColors)
     return 0;
 
   fvwm_msg(ERR,"FvwmErrorHandler","*** internal error ***");
@@ -2059,8 +1759,8 @@ int FvwmErrorHandler(Display *dpy, XErrorEvent *event)
 
 static void usage(void)
 {
-  fprintf(stderr,"FVWM version %s Usage:\n\n",VERSION);
-  fprintf(stderr,"  %s [-d dpy] [-debug] [-f config_cmd] [-s] [-blackout] [-version] [-h] [-replace] [-clientId id] [-restore file] [-visualId id] [-visual class]\n\n",g_argv[0]);
+  fprintf(stderr,"\nFVWM version %s Usage:\n\n",VERSION);
+  fprintf(stderr,"  %s [-d dpy] [-debug] [-f config_file] [-cmd config_cmd] [-s] [-version] [-h] [-replace] [-clientId id] [-restore file] [-visualId id] [-visual class]\n\n",g_argv[0]);
   exit(1);
 }
 
@@ -2096,6 +1796,11 @@ void SetMWM_INFO(Window window)
     long flags;
     Window win;
   }  motif_wm_info;
+  static char set_yorn='n';
+
+  if(set_yorn=='y') {
+    return;
+  }
 
   if (Scr.bo.ModalityIsEvil)
   {
@@ -2103,9 +1808,9 @@ void SetMWM_INFO(Window window)
      * broken handling of modal dialogs */
     motif_wm_info.flags = 2;
     motif_wm_info.win = window;
-
     XChangeProperty(dpy,Scr.Root,_XA_MOTIF_WM,_XA_MOTIF_WM,32,
 		    PropModeReplace,(unsigned char *)&motif_wm_info,2);
+    set_yorn='y';
   }
 }
 
@@ -2253,7 +1958,7 @@ static void setVersionInfo(void)
   /* Set version information string */
   sprintf(version_str, "FVWM version %s compiled on %s at %s",
           VERSION, __DATE__, __TIME__);
-  Fvwm_VersionInfo = strdup(version_str);
+  Fvwm_VersionInfo = safestrdup(version_str);
 
 #ifdef HAVE_READLINE
   strcat(support_str, " ReadLine,");
@@ -2267,24 +1972,27 @@ static void setVersionInfo(void)
 #ifdef XPM
   strcat(support_str, " XPM,");
 #endif
-/* no sence to report Imlib and not report GTK; both are used in FvwmGtk only
-#ifdef IMLIB
-  strcat(support_str, " Imlib,");
-#endif
-*/
 #ifdef GNOME
   strcat(support_str, " GNOME WM hints,");
 #endif
+  if (FHaveShapeExtension)
+    strcat(support_str, " Shape,");
 #ifdef SESSION
   strcat(support_str, " SM,");
 #endif
+#ifdef I18N_MB
+  strcat(support_str, " Multibyte,");
+#endif
+#ifdef HAVE_XINERAMA
+  strcat(support_str, " Xinerama,");
+#endif
 
   support_len = strlen(support_str);
-  if (support_len)
+  if (support_len > 0)
   {
     /* strip last comma */
     support_str[support_len - 1] = '\0';
-    Fvwm_SupportInfo = strdup(CatString2("with support for:", support_str));
+    Fvwm_SupportInfo = safestrdup(CatString2("with support for:", support_str));
   }
   else
     Fvwm_SupportInfo = "with no optional feature support";
