@@ -1223,12 +1223,15 @@ static void GetXPMFile(FvwmWindow *tmp_win)
  ****************************************************************************/
 void GetIconWindow(FvwmWindow *tmp_win)
 {
+  unsigned int w;
+  unsigned int h;
+  unsigned int bw;
+
   /* We are guaranteed that wmhints is non-null when calling this
    * routine */
   if (XGetGeometry(
 	dpy, tmp_win->wmhints->icon_window, &JunkRoot, &JunkX, &JunkY,
-	(unsigned int *)&tmp_win->icon_p_width,
-	(unsigned int *)&tmp_win->icon_p_height, &JunkBW, &JunkDepth) == 0)
+	&w, &h, &bw, &JunkDepth) == 0)
   {
     fvwm_msg(ERR,"GetIconWindow", "Window '%s' has a bad icon window!"
 	     " Ignoring icon window.",
@@ -1238,8 +1241,9 @@ void GetIconWindow(FvwmWindow *tmp_win)
     tmp_win->wmhints->flags &= ~IconWindowHint;
     return;
   }
-  tmp_win->icon_p_width += JunkBW<<1;
-  tmp_win->icon_p_height += JunkBW<<1;
+  tmp_win->icon_border_width = bw;
+  tmp_win->icon_p_width = w + 2 * bw;
+  tmp_win->icon_p_height = h + 2 * bw;
   /*
    * Now make the new window the icon window for this window,
    * and set it up to work as such (select for key presses
@@ -1323,7 +1327,7 @@ void GetIconBitmap(FvwmWindow *tmp_win)
  ***********************************************************************/
 void DeIconify(FvwmWindow *tmp_win)
 {
-  FvwmWindow *t,*tmp;
+  FvwmWindow *t, *tmp, *ofw;
   FvwmWindow *sf = get_focus_window();
 
   if (!tmp_win)
@@ -1334,14 +1338,19 @@ void DeIconify(FvwmWindow *tmp_win)
     SET_ICONIFY_AFTER_MAP(tmp_win, 0);
     return;
   }
-  while (IS_ICONIFIED_BY_PARENT(tmp_win))
+  for (ofw = NULL; tmp_win != ofw && IS_ICONIFIED_BY_PARENT(tmp_win); )
   {
-    for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
+    t = get_transientfor_fvwmwindow(tmp_win);
+    if (t != NULL)
     {
-      if (t != tmp_win && tmp_win->transientfor == t->w)
-	tmp_win = t;
+      ofw = tmp_win;
+      tmp_win = t;
     }
   }
+  if (IS_ICONIFIED_BY_PARENT(tmp_win))
+  {
+    SET_ICONIFIED_BY_PARENT(tmp_win, 0);
+  }
 
   /* AS dje  RaiseWindow(tmp_win); */
 
