@@ -22,6 +22,7 @@
 #define ICON_HEIGHT(t) ((t)->icon_font.height + 6)
 #endif
 
+int get_visible_icon_window_count(FvwmWindow *tmp_win);
 void AutoPlaceIcon(FvwmWindow *);
 void RedoIconName(FvwmWindow *);
 void DrawIconWindow(FvwmWindow *);
@@ -31,7 +32,6 @@ void GetIconBitmap(FvwmWindow *tmp_win);
 void Iconify(FvwmWindow *, int, int);
 void DeIconify(FvwmWindow *);
 void SetMapStateProp(FvwmWindow *, int);
-void iconify_function(F_CMD_ARGS);
 
 
 #endif /* _ICONS_ */
