@@ -53,6 +53,7 @@
 #include "screen.h"
 #include "module_interface.h"
 #include "libs/Colorset.h"
+#include "libs/FScreen.h"
 
 extern unsigned long *PipeMask;                /* in module.c */
 
@@ -153,8 +154,7 @@ static struct moduleInfoList *AddToModList(char *tline)
 /**************************************************************/
 /* delete from module configuration                           */
 /**************************************************************/
-void DestroyModConfig(XEvent *eventp,Window junk,FvwmWindow *tmp_win,
-                      unsigned long context, char *action,int* Module)
+void CMD_DestroyModuleConfig(F_CMD_ARGS)
 {
   struct moduleInfoList *current, *next, *prev;
   char *info;   /* info to be deleted - may contain wildcards */
@@ -218,8 +218,13 @@ void DestroyModConfig(XEvent *eventp,Window junk,FvwmWindow *tmp_win,
   free(info);
 }
 
-void SendDataToModule(XEvent *eventp,Window w,FvwmWindow *tmp_win,
-		      unsigned long context, char *action, int *Module)
+static void send_xinerama_state(int modnum)
+{
+  SendName(modnum, M_CONFIG_INFO, 0, 0, 0, FScreenGetConfiguration());
+  return;
+}
+
+void CMD_Send_ConfigInfo(F_CMD_ARGS)
 {
   struct moduleInfoList *t;
   char *message, msg2[32];
@@ -252,6 +257,7 @@ void SendDataToModule(XEvent *eventp,Window w,FvwmWindow *tmp_win,
     SendName(*Module,M_CONFIG_INFO,0,0,0,msg2);
   }
 #endif
+  send_xinerama_state(*Module);
   /* now dump the colorsets (0 first as others copy it) */
   for (n = 0; n < nColorsets; n++)
     SendName(*Module, M_CONFIG_INFO, 0, 0, 0, DumpColorset(n, &Colorset[n]));
