@@ -16,8 +16,7 @@
 #ifndef MODCONF_H
 #define MODCONF_H
 
-void DestroyModConfig(F_CMD_ARGS);
+
 void ModuleConfig(char *action);
-void SendDataToModule(F_CMD_ARGS);
 
 #endif /* MODCONF_H */
