@@ -31,12 +31,15 @@
 #include <stdio.h>
 #include <stdarg.h>
 #include <errno.h>
+#include <time.h>
 
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
 #endif
 
 #include "libs/fvwmlib.h"
+#include "libs/Colorset.h"
+#include "libs/FScreen.h"
 #include "fvwm.h"
 #include "externs.h"
 #include "cursor.h"
@@ -48,7 +51,6 @@
 #include "read.h"
 #include "events.h"
 #include "fvwmsignal.h"
-#include "libs/Colorset.h"
 
 /*
  * Use POSIX behaviour if we can, otherwise use SysV instead
@@ -371,13 +373,13 @@ int executeModuleDesperate(F_CMD_ARGS)
   return do_execute_module(F_PASS_ARGS, True);
 }
 
-void executeModule(F_CMD_ARGS)
+void CMD_Module(F_CMD_ARGS)
 {
   do_execute_module(F_PASS_ARGS, False);
   return;
 }
 
-void executeModuleSync(F_CMD_ARGS)
+void CMD_ModuleSynchronous(F_CMD_ARGS)
 {
   extern char *ModuleFinished;		/* defined in libs/Module.c */
   int sec = 0;
@@ -470,7 +472,7 @@ void executeModuleSync(F_CMD_ARGS)
       timeout.tv_usec = 1;  /* use 1 usec timeout in select */
       num_fd = fvwmSelect(fd_width, &in_fdset, &out_fdset, 0, &timeout);
     }
-    while (num_fd < 0);
+    while ( (num_fd < 0) && !isTerminated );
 
     /* Exit if we have received a "terminate" signal */
     if ( isTerminated ) break;
@@ -624,20 +626,34 @@ void ExecuteModuleCommand(Window w, int module, char *text)
   /* Query the pointer, the pager-drag-out feature doesn't work properly.*/
   /* This is OK now that the Pager uses "Move pointer" */
   /* A real fix would be for the modules to pass the button press coords */
-  XQueryPointer(dpy, Scr.Root, &JunkRoot, &JunkChild, &JunkX,&JunkY,
-		&Event.xbutton.x_root,&Event.xbutton.y_root,&JunkMask);
+  if (XQueryPointer(dpy, Scr.Root, &JunkRoot, &JunkChild, &JunkX,&JunkY,
+		    &Event.xbutton.x_root,&Event.xbutton.y_root,&JunkMask) ==
+      False)
+  {
+    /* pointer is not on this screen */
+    /* If a module does XUngrabPointer(), it can now get proper Popups */
+    Event.xbutton.window = Scr.Root;
+    Event.xbutton.button = 1;
+    Event.xbutton.subwindow = None;
+    tmp_win = NULL;
+    ButtonWindow = NULL;
+  }
+  else
+  {
+    Event.xbutton.window = w;
+    Event.xbutton.button = 1;
+    Event.xbutton.subwindow = None;
+    Context = GetContext(tmp_win,&Event,&w);
+    ButtonWindow = tmp_win;
+  }
   /* If a module does XUngrabPointer(), it can now get proper Popups */
   if(StrEquals(text, "popup"))
     Event.xbutton.type = ButtonPress;
   else
     Event.xbutton.type = ButtonRelease;
-  Event.xbutton.window = w;
-  Event.xbutton.button = 1;
   Event.xbutton.x = 0;
   Event.xbutton.y = 0;
-  Event.xbutton.subwindow = None;
   Context = GetContext(tmp_win,&Event,&w);
-  ButtonWindow = tmp_win;
   old_execute_function(text, tmp_win, &Event, Context, module, 0, NULL);
   ButtonWindow = NULL;
 }
@@ -706,7 +722,7 @@ static void KillModuleByName(char *name
   return;
 }
 
-void module_zapper(F_CMD_ARGS)
+void CMD_KillModule(F_CMD_ARGS)
 {
   char *module;
 #ifndef WITHOUT_KILLMODULE_ALIAS_SUPPORT
@@ -733,6 +749,9 @@ make_vpacket(unsigned long *body, unsigned long event_type,
   extern Time lastTimestamp;
   unsigned long *bp = body;
 
+  /* truncate long packets */
+  if (num > FvwmPacketMaxSize)
+    num = FvwmPacketMaxSize;
   *(bp++) = START_FLAG;
   *(bp++) = event_type;
   *(bp++) = num+FvwmPacketHeaderSize;
@@ -746,7 +765,7 @@ make_vpacket(unsigned long *body, unsigned long event_type,
 
 
 
-/* ===========================================================
+/*************************************************************
     RBW - 04/16/1999 - new packet builder for GSFR --
     Arguments are pairs of lengths and argument data pointers.
     RBW - 05/01/2000 -
@@ -755,7 +774,7 @@ make_vpacket(unsigned long *body, unsigned long event_type,
     a special hack to accommodate the old CONFIGARGSNEW
     technique of sending the args for the M_CONFIGURE_WINDOW
     packet.
-   =========================================================== */
+**************************************************************/
 static unsigned long
 make_new_vpacket(unsigned char *body, unsigned long event_type,
 		 unsigned long num, va_list ap)
@@ -840,9 +859,9 @@ BroadcastPacket(unsigned long event_type, unsigned long num_datum, ...)
 }
 
 
-/* ============================================================
+/* ************************************************************
     RBW - 04/16/1999 - new style packet senders for GSFR --
-   ============================================================ */
+   ************************************************************ */
 static void SendNewPacket(int module, unsigned long event_type,
 			  unsigned long num_datum, ...)
 {
@@ -967,7 +986,7 @@ static void BroadcastNewPacket(unsigned long event_type,
 
 
 
-/* ===============================================================
+/****************************************************************
     RBW - 04/16/1999 - new version for GSFR --
         - args are now pairs:
           - length of arg data
@@ -976,7 +995,7 @@ static void BroadcastNewPacket(unsigned long event_type,
         - the 9th field, where flags used to be, is temporarily left
         as a dummy to preserve alignment of the other fields in the
         old packet: we should drop this before the next release.
-   =============================================================== */
+*****************************************************************/
 #define CONFIGARGSNEW(_t) 25,\
 	    (unsigned long)(sizeof(unsigned long)),\
             &(*(_t))->w,\
@@ -1078,8 +1097,12 @@ make_named_packet(int *len, unsigned long event_type, const char *name,
   va_list ap;
 
   /* Packet is the header plus the items plus enough items to hold the name
-     string.  */
-  *len = FvwmPacketHeaderSize + num + (strlen(name) / sizeof(unsigned long)) + 1;
+   * string. */
+  *len =
+    FvwmPacketHeaderSize + num + (strlen(name) / sizeof(unsigned long)) + 1;
+  /* truncate long packets */
+  if (*len > FvwmPacketMaxSize)
+    *len = FvwmPacketMaxSize;
 
   body = (unsigned long *)safemalloc(*len * sizeof(unsigned long));
   body[*len-1] = 0; /* Zero out end of memory to avoid uninit memory access. */
@@ -1088,11 +1111,15 @@ make_named_packet(int *len, unsigned long event_type, const char *name,
   make_vpacket(body, event_type, num, ap);
   va_end(ap);
 
-  strcpy((char *)&body[FvwmPacketHeaderSize+num], name);
+  strncpy((char *)&body[FvwmPacketHeaderSize+num], name,
+	  (*len - FvwmPacketHeaderSize - num)*sizeof(unsigned long) - 1);
   body[2] = *len;
 
-  /* DB(("Packet (%lu): %lu %lu %lu `%s'", *len,
-       body[FvwmPacketHeaderSize], body[FvwmPacketHeaderSize+1], body[FvwmPacketHeaderSize+2], name)); */
+#if 0
+  DB(("Packet (%lu): %lu %lu %lu `%s'", *len,
+      body[FvwmPacketHeaderSize], body[FvwmPacketHeaderSize+1],
+      body[FvwmPacketHeaderSize+2], name));
+#endif
 
   return (body);
 }
@@ -1192,11 +1219,36 @@ void BroadcastColorset(int n)
 }
 
 
+/**********************************************************************
+ * Broadcasts a string to all modules as M_CONFIG_INFO.
+ **********************************************************************/
+void BroadcastConfigInfoString(char *string)
+{
+  int i;
+
+  for (i = 0; i < npipes; i++)
+  {
+    /* just a quick check to save us lots of overhead */
+    if (pipeOn[i] >= 0)
+      SendName(i, M_CONFIG_INFO, 0, 0, 0, string);
+  }
+}
+
+
+/**********************************************************************
+ * Broadcasts the state of Xinerama support to all modules as M_CONFIG_INFO.
+ **********************************************************************/
+void broadcast_xinerama_state(void)
+{
+  BroadcastConfigInfoString((char *)FScreenGetConfiguration());
+  return;
+}
+
+
 /*
 ** send an arbitrary string to all instances of a module
 */
-void SendStrToModule(XEvent *eventp,Window junk,FvwmWindow *tmp_win,
-                     unsigned long context, char *action,int* Module)
+void CMD_SendToModule(F_CMD_ARGS)
 {
   char *module,*str;
   unsigned long data0, data1, data2;
@@ -1207,10 +1259,9 @@ void SendStrToModule(XEvent *eventp,Window junk,FvwmWindow *tmp_win,
   /* UngrabEm(); */
   if (!action)
     return;
-  GetNextToken(action,&module);
+  str = GetNextToken(action, &module);
   if (!module)
     return;
-  str = strdup(action + strlen(module) + 1);
 
   if (tmp_win) {
     /* Modules may need to know which window this applies to */
@@ -1236,7 +1287,6 @@ void SendStrToModule(XEvent *eventp,Window junk,FvwmWindow *tmp_win,
   }
 
   free(module);
-  free(str);
 }
 
 
@@ -1299,24 +1349,48 @@ void PositiveWrite(int module, unsigned long *ptr, int size)
 
       do
       {
-	timeout.tv_sec = moduleTimeout;
-	timeout.tv_usec = 0;
-	FD_ZERO(&readSet);
-	FD_SET(channel, &readSet);
+        timeout.tv_sec = moduleTimeout;
+        timeout.tv_usec = 0;
+        FD_ZERO(&readSet);
+        FD_SET(channel, &readSet);
+
+        /* Wait for input to arrive on just one descriptor, with a timeout */
+        /* (fvwmSelect <= 0) or read() returning wrong size is bad news */
+        rc = fvwmSelect(channel + 1, &readSet, NULL, NULL, &timeout);
+        /* retry if select() failed with EINTR */
+      } while ((rc < 0) && !isTerminated && (errno == EINTR));
+
+      if ( isTerminated ) break;
 
-	/* Wait for input to arrive on just one descriptor, with a timeout */
-	/* (fvwmSelect <= 0) or read() returning wrong size is bad news */
-	rc = fvwmSelect(channel + 1, &readSet, NULL, NULL, &timeout);
-	/* retry if select() failed with EINTR */
-      } while (rc < 0 && errno == EINTR);
       if (rc <= 0 || read(channel, &targetWindow, sizeof(targetWindow))
-	  != sizeof(targetWindow))
+           != sizeof(targetWindow))
       {
+        char *name;
+
+        if (pipeName[module] != NULL)
+        {
+#ifndef WITHOUT_KILLMODULE_ALIAS_SUPPORT
+          if (pipeAlias[module] != NULL)
+          {
+            name = CatString3(pipeName[module], " ", pipeAlias[module]);
+          }
+          else
+#endif
+          {
+            name = pipeName[module];
+          }
+        }
+        else
+        {
+          name = "(null)";
+        }
+
         /* Doh! Something has gone wrong - get rid of the offender!! */
         fvwm_msg(ERR, "PositiveWrite",
-                 "Failed to read descriptor:\n"
+                 "Failed to read descriptor from '%s':\n"
                  "- data available=%c\n"
                  "- terminate signal=%c\n",
+                 name,
                  (FD_ISSET(channel, &readSet) ? 'Y' : 'N'),
                  isTerminated ? 'Y' : 'N');
         KillModule(module);
@@ -1460,7 +1534,7 @@ static void AddToCommandQueue(Window window, int module, char *command)
 
   new->window = window;
   new->module = module;
-  new->command = strdup(command);
+  new->command = safestrdup(command);
   new->next = NULL;
 
   DBUG("AddToCommandQueue", command);
@@ -1504,8 +1578,7 @@ void ExecuteCommandQueue(void)
   }
 }
 
-void send_list_func(XEvent *eventp, Window w, FvwmWindow *tmp_win,
-                    unsigned long context, char *action, int *Module)
+void CMD_Send_WindowList(F_CMD_ARGS)
 {
   FvwmWindow *t;
 
@@ -1528,8 +1601,8 @@ void send_list_func(XEvent *eventp, Window w, FvwmWindow *tmp_win,
       {
 	SendPacket(*Module, M_FOCUS_CHANGE, 5,
                    0, 0, (unsigned long)True,
-                   GetColor("White"),
-                   GetColor("Black"));
+                   GetColor(DEFAULT_FORE_COLOR),
+                   GetColor(DEFAULT_BACK_COLOR));
       }
       if (Scr.DefaultIcon != NULL)
 	SendName(*Module, M_DEFAULTICON, 0, 0, 0, Scr.DefaultIcon);
@@ -1579,8 +1652,8 @@ void send_list_func(XEvent *eventp, Window w, FvwmWindow *tmp_win,
       {
 	  BroadcastPacket(M_FOCUS_CHANGE, 5,
                           0, 0, (unsigned long)True,
-			  GetColor("White"),
-			  GetColor("Black"));
+			  GetColor(DEFAULT_FORE_COLOR),
+			  GetColor(DEFAULT_BACK_COLOR));
       }
       else
       {
@@ -1596,7 +1669,7 @@ void send_list_func(XEvent *eventp, Window w, FvwmWindow *tmp_win,
     }
 }
 
-void set_mask_function(F_CMD_ARGS)
+void CMD_set_mask(F_CMD_ARGS)
 {
   int val = 0;
 
@@ -1604,7 +1677,7 @@ void set_mask_function(F_CMD_ARGS)
   PipeMask[*Module] = (unsigned long)val;
 }
 
-void setSyncMaskFunc(F_CMD_ARGS)
+void CMD_set_sync_mask(F_CMD_ARGS)
 {
   int val = 0;
 
@@ -1612,7 +1685,7 @@ void setSyncMaskFunc(F_CMD_ARGS)
   SyncMask[*Module] = (unsigned long)val;
 }
 
-void setNoGrabMaskFunc(F_CMD_ARGS)
+void CMD_set_nograb_mask(F_CMD_ARGS)
 {
   int val = 0;
 
@@ -1625,9 +1698,10 @@ void setNoGrabMaskFunc(F_CMD_ARGS)
  */
 char *skipModuleAliasToken(const char *string)
 {
-  #define is_valid_first_alias_char(ch) (isalpha(ch))
-  #define is_valid_alias_char(ch) (is_valid_first_alias_char(ch) \
-    || isalnum(ch) || (ch) == '-' || (ch) == '.')
+#define is_valid_first_alias_char(ch) (isalpha(ch) || (ch) == '/')
+#define is_valid_alias_char(ch) (is_valid_first_alias_char(ch) \
+    || isalnum(ch) || (ch) == '-' || (ch) == '.' || (ch) == '/')
+
   if (is_valid_first_alias_char(*string))
   {
     int len = 1;
