@@ -29,6 +29,37 @@
 #include "Parse.h"
 
 
+/* This function escapes all occurences of the characters in the string qchars
+ * in the string as with a preceding echar character.  The resulting string is
+ * returned in a malloced memory area. */
+char *EscapeString(char *s, const char *qchars, char echar)
+{
+  char *t;
+  char *ret;
+  int len;
+
+  for (len = 1, t = s; *t ; t++, len++)
+  {
+    if (strchr(qchars, *t) != NULL)
+    {
+      len++;
+    }
+  }
+  ret = (char *)safemalloc(len);
+  for (t = ret; *s; s++, t++)
+  {
+    if (strchr(qchars, *s) != NULL)
+    {
+      *t = echar;
+      t++;
+    }
+    *t = *s;
+  }
+  *t = 0;
+
+  return ret;
+}
+
 /* If the string s begins with a quote chracter SkipQuote returns a pointer
  * to the first unquoted character or to the final '\0'. If it does not, a
  * pointer to the next character in the string is returned.
@@ -67,31 +98,31 @@ char *SkipQuote(char *s, const char *qlong, const char *qstart,
   if (*s == '\\' && s[1] != 0)
     return s+2;
   else if (*qlong && (t = strchr(qlong, *s)))
-    {
-      char c = *t;
+  {
+    char c = *t;
 
-      s++;
-      while(*s && *s != c)
-	{
-	  /* Skip over escaped text, ie \quote */
-	  if(*s == '\\' && *(s+1) != 0)
-	    s++;
-	  s++;
-	}
-      if(*s == c)
+    s++;
+    while(*s && *s != c)
+    {
+      /* Skip over escaped text, ie \quote */
+      if(*s == '\\' && *(s+1) != 0)
 	s++;
-      return s;
+      s++;
     }
+    if(*s == c)
+      s++;
+    return s;
+  }
   else if (*qstart && (t = strchr(qstart, *s)))
-    {
-      char c = *((t - qstart) + qend);
+  {
+    char c = *((t - qstart) + qend);
 
-      while (*s && *s != c)
-	s = SkipQuote(s, qlong, "", "");
-      return (*s == *t) ? ++s : s;
-    }
-  else
-    return ++s;
+    while (*s && *s != c)
+      s = SkipQuote(s, qlong, "", "");
+    return (*s == c) ? ++s : s;
+  }
+
+  return ++s;
 }
 
 /* Returns a string up to the first character from the string delims in a
@@ -146,39 +177,60 @@ char *SkipSpaces(char *indata, char *spaces, int snum)
 static char *CopyToken(char *src, char *dest, char *spaces, int snum,
 		       char *delims, int dnum, char *out_delim)
 {
+  int len = 0;
+
   while ( (*src != 0) && !(isspace((unsigned char)*src) ||
 			   (snum && strchr(spaces, *src)) ||
 			   (dnum && strchr(delims, *src))))
+  {
+    /* Check for quoted text */
+    if (IsQuote(*src))
     {
-      /* Check for qouted text */
-      if (IsQuote(*src))
-	{
-	  char c = *src;
+      char c = *src;
 
+      src++;
+      while((*src != c)&&(*src != 0))
+      {
+	if((*src == '\\' && *(src+1) != 0))
+	  /* Skip over backslashes */
 	  src++;
-	  while((*src != c)&&(*src != 0))
-	    {
-	      if((*src == '\\' && *(src+1) != 0))
-		/* Skip over backslashes */
-		src++;
-	      *(dest++) = *(src++);
-	    }
-	  if(*src == c)
-	    src++;
-	}
-      else
+	if (len < MAX_TOKEN_LENGTH - 1)
 	{
-	  if((*src == '\\' && *(src+1) != 0))
-	    /* Skip over backslashes */
-	    src++;
+	  len++;
 	  *(dest++) = *(src++);
 	}
+	else
+	{
+	  /* token too long, just skip rest of token */
+	  src++;
+	}
+      }
+      if(*src == c)
+	src++;
     }
+    else
+    {
+      if((*src == '\\' && *(src+1) != 0))
+	/* Skip over backslashes */
+	src++;
+      if (len < MAX_TOKEN_LENGTH - 1)
+      {
+	len++;
+	*(dest++) = *(src++);
+      }
+      else
+      {
+	/* token too long, just skip rest of token */
+	src++;
+      }
+    }
+  }
   if (out_delim)
     *out_delim = *src;
   *dest = 0;
   if (*src != 0)
     src++;
+
   return src;
 }
 
@@ -331,7 +383,7 @@ char *DoGetNextToken(char *indata, char **token, char *spaces, char *delims,
   if (tmptok == NULL)
     *token = NULL;
   else
-    *token = strdup(tmptok);
+    *token = safestrdup(tmptok);
 
   return end;
 }
@@ -411,8 +463,9 @@ char *GetModuleResource(char *indata, char **resource, char *module_name)
  * ret_suffixnum (0 = no suffix, 1 = first suffix in suffixlist ...).
  *
  **************************************************************************/
-int GetSuffixedIntegerArguments(char *action, char **ret_action, int retvals[],
-				int num, char *suffixlist, int ret_suffixnum[])
+int GetSuffixedIntegerArguments(
+  char *action, char **ret_action, int *retvals, int num, char *suffixlist,
+  int *ret_suffixnum)
 {
   int i;
   int j;
@@ -474,7 +527,7 @@ int GetSuffixedIntegerArguments(char *action, char **ret_action, int retvals[],
  * it is big enough before calling this function.
  *
  **************************************************************************/
-int SuffixToPercentValue(int value, int suffix, int unit_table[])
+int SuffixToPercentValue(int value, int suffix, int *unit_table)
 {
   return (value * unit_table[suffix]) / 100;
 }
@@ -486,10 +539,10 @@ int SuffixToPercentValue(int value, int suffix, int unit_table[])
  * If ret_action is non-NULL, a pointer to the next token is returned there.
  *
  **************************************************************************/
-int GetIntegerArguments(char *action, char **ret_action, int retvals[],int num)
+int GetIntegerArguments(char *action, char **ret_action, int *retvals,int num)
 {
-  return GetSuffixedIntegerArguments(action, ret_action, retvals, num, NULL,
-				     NULL);
+  return GetSuffixedIntegerArguments(
+    action, ret_action, retvals, num, NULL, NULL);
 }
 
 
@@ -507,7 +560,7 @@ int GetIntegerArguments(char *action, char **ret_action, int retvals[],int num)
  * in token after the match.
  *
  **************************************************************************/
-int GetTokenIndex(char *token, char *list[], int len, char **next)
+int GetTokenIndex(char *token, char **list, int len, char **next)
 {
   int i;
   int l;
@@ -546,7 +599,7 @@ int GetTokenIndex(char *token, char *list[], int len, char **next)
  * token (just like the return value of GetNextToken).
  *
  **************************************************************************/
-char *GetNextTokenIndex(char *action, char *list[], int len, int *index)
+char *GetNextTokenIndex(char *action, char **list, int len, int *index)
 {
   char *token;
   char *next;
@@ -654,8 +707,8 @@ int GetTwoPercentArguments(char *action, int *val1, int *val2, int *val1_unit,
  * token matches none of these strings the default_ret value is returned and
  * the action itself is passed back in ret_action. If the no_toggle flag is
  * non-zero, the "toggle" string is handled as no match. */
-int ParseToggleArgument(char *action, char **ret_action, int default_ret,
-			char no_toggle)
+int ParseToggleArgument(
+  char *action, char **ret_action, int default_ret, char no_toggle)
 {
   int index;
   int rc;
