@@ -21,6 +21,12 @@ char *CatString3(const char *a, const char *b, const char *c);
 void CopyString(char **dest, const char *source);
 
 
+/**
+ * Like CopyString, but strips leading and trailing (double) quotes if any.
+ **/
+void CopyStringWithQuotes(char **dest, const char *src);
+
+
 /**
  * Copy string into newly-malloced memory, stripping leading and
  * trailing spaces.  The difference between this and CopyString()
