@@ -645,6 +645,7 @@ void SetTransparentBackground(button_info *ub,int w,int h)
       Dpy, MyWindow, FShapeBounding, 0, 0, pmap_mask, FShapeSet);
     XFreePixmap(Dpy, pmap_mask);
     XDestroyRegion(shape_r);
+    XFlush(Dpy);
   }
 
   return;
@@ -839,7 +840,7 @@ int main(int argc, char **argv)
   /* parse the geometry string */
   if (geom_option_argc != 0)
   {
-    parse_window_geometry(argv[geom_option_argc]);
+    parse_window_geometry(argv[geom_option_argc], 0);
     has_button_geometry = 0;
   }
 
@@ -1063,14 +1064,14 @@ void Loop(void)
 	  SetButtonSize(UberButton,Width,Height);
 	  button=-1;
 	  ub=UberButton;
-	  if (FShapesSupported && UberButton->c->flags & b_TransBack)
-	  {
-	    SetTransparentBackground(UberButton, Width, Height);
-	  }
 	  for (i = 0; i < nColorsets; i++)
 	    change_colorset(i);
 	  while(NextButton(&ub,&b,&button,0))
 	    MakeButton(b);
+	  if (FShapesSupported && UberButton->c->flags & b_TransBack)
+	  {
+	    SetTransparentBackground(UberButton, Width, Height);
+	  }
 	  RedrawWindow(NULL);
 	}
       }
