@@ -157,6 +157,13 @@ typedef struct _line {
   Item **items;                         /* ptr to array of items */
 } Line;
 
+/* Need a struct for the pointer color.
+   Since colors are unsigned, we need a flag to indicate whether
+   the color was ever set. */
+typedef struct _ptrc {
+  XColor pointer_color;
+  char used;
+} Ptr_color;
 
 /* There is one "form". Start of attempt to change that. */
 typedef struct _form {
@@ -167,6 +174,7 @@ typedef struct _form {
   int grab_server;                      /* set during parse used on display */
   int server_grabbed;                   /* first time switch */
   int gx, gy, have_geom;
+  int xneg, yneg;
   int warp_pointer;
   int max_width, total_height;          /* frame size */
   unsigned long screen_background;
@@ -184,15 +192,20 @@ typedef struct _form {
   char *expand_buffer;                  /* buffer to expand commands in */
   int expand_buffer_size;               /* current size */
   char have_env_var;                    /* at least one env var on cmd line */
+  Cursor pointer[3];
+  Ptr_color p_c[6];                     /* fg/bg for pointers, used flag */
+  int activate_on_press;                /* activate button on press */
 } Form;
 
+enum {input_pointer,button_pointer,button_in_pointer};
+enum {input_fore,input_back,
+      button_fore,button_back,
+      button_in_fore,button_in_back};
+
 extern Form cur_form;                   /* current form */
 #define CF cur_form                     /* I may want to undo this... */
 
   /* Globals: */
-/* These aren't customizable as to shape or color, so for now, they aren't
-   in the "form" structure */
-extern Cursor xc_ibeam, xc_hand;
 
 /* Link list roots */
 extern Item *root_item_ptr;             /* pointer to root of item list */
