@@ -18,11 +18,14 @@
 #include <stdlib.h>
 #include "FvwmIconMan.h"
 #include "readconfig.h"
-#include "libs/defaults.h"
+#include <libs/defaults.h>
+#include <libs/fvwmlib.h>
+#include <libs/FScreen.h>
+#include <libs/FShape.h>
 #include <libs/Module.h>
 
 static char const rcsid[] =
-  "$Id: readconfig.c,v 1.32 2001/06/23 10:32:16 domivogt Exp $";
+  "$Id: readconfig.c,v 1.38.2.3 2003/01/24 13:58:15 domivogt Exp $";
 
 /************************************************************************
  *
@@ -897,6 +900,12 @@ static int GetConfigLineWrapper (int *fd, char **tline)
     if (strncasecmp(*tline, "Colorset", 8) == 0) {
          LoadColorset(&(*tline)[8]);
     }
+    else if (strncasecmp(*tline, XINERAMA_CONFIG_STRING,
+			 sizeof(XINERAMA_CONFIG_STRING) - 1) == 0)
+    {
+      FScreenConfigureModule(
+	(*tline) + sizeof(XINERAMA_CONFIG_STRING) - 1);
+    }
     temp = strchr (*tline, '\n');
     if (temp) {
       *temp = '\0';
@@ -919,7 +928,8 @@ static char *read_next_cmd (ReadOption flag)
 {
   static ReadOption status;
   static char *buffer;
-  static char *retstring, displaced, *cur_pos;
+  static char *retstring, *cur_pos;
+  char *end_pos;
 
   retstring = NULL;
   if (flag != READ_LINE && !(flag & status))
@@ -933,13 +943,13 @@ static char *read_next_cmd (ReadOption flag)
       if (!strncasecmp (Module, cur_pos, ModuleLen)) {
         retstring = cur_pos;
         cur_pos += ModuleLen;
-        displaced = *cur_pos;
-        if (displaced == '*')
+
+        if (*cur_pos == '*')
+          cur_pos++;
+        skip_space(&cur_pos);
+
+        if (isalnum(*cur_pos))
           status = READ_OPTION;
-        else if (displaced == '\0')
-          status = READ_LINE;
-        else if (iswhite (displaced))
-          status = READ_ARG;
         else
           status = READ_LINE;
         break;
@@ -948,42 +958,43 @@ static char *read_next_cmd (ReadOption flag)
     break;
 
   case READ_OPTION:
-    *cur_pos = displaced;
-    retstring = ++cur_pos;
+    retstring = cur_pos;
     while (*cur_pos != '*' && !iswhite (*cur_pos))
       cur_pos++;
-    displaced = *cur_pos;
-    *cur_pos = '\0';
-    if (displaced == '*')
+    end_pos = cur_pos;
+
+    if (*cur_pos == '*')
+      cur_pos++;
+    skip_space(&cur_pos);
+
+    if (!*cur_pos)
+      status = READ_LINE;
+    else if (isdigit(*retstring) || !strncmp(retstring, "transient", 9))
       status = READ_OPTION;
-    else if (displaced == '\0')
-      status = READ_LINE;
-    else if (iswhite (displaced))
+    else if (*cur_pos)
       status = READ_ARG;
-    else
-      status = READ_LINE;
+
+    *end_pos = '\0';
     break;
 
   case READ_ARG:
-    *cur_pos = displaced;
-    skip_space (&cur_pos);
     retstring = cur_pos;
     while (!iswhite (*cur_pos))
       cur_pos++;
-    displaced = *cur_pos;
-    *cur_pos = '\0';
-    if (displaced == '\0')
-      status = READ_LINE;
-    else if (iswhite (displaced))
+    end_pos = cur_pos;
+
+    skip_space(&cur_pos);
+
+    if (*cur_pos)
       status = READ_ARG;
     else
       status = READ_LINE;
+
+    *end_pos = '\0';
     break;
 
   case READ_REST_OF_LINE:
     status = READ_LINE;
-    *cur_pos = displaced;
-    skip_space (&cur_pos);
     retstring = cur_pos;
     break;
   }
@@ -1384,17 +1395,41 @@ void read_in_resources (char *file)
 	ConsoleDebug (CONFIG, "Setting followfocus to: %d\n", i);
 	SET_MANAGER (manager, followFocus, i);
       }
+      else if (!strcasecmp (option1, "showonlyicons")) {
+       p = read_next_cmd (READ_ARG);
+       if (!p) {
+         ConsoleMessage ("Bad line: %s\n", current_line);
+         ConsoleMessage ("Need argument to showonlyicons\n");
+         continue;
+       }
+       if (!strcasecmp (p, "true")) {
+         i = 1;
+       }
+       else if (!strcasecmp (p, "false")) {
+         i = 0;
+       }
+       else {
+         ConsoleMessage ("Bad line: %s\n", current_line);
+         ConsoleMessage ("What is this: %s?\n", p);
+         continue;
+       }
+       ConsoleDebug (CONFIG, "Setting showonlyicons to: %d\n", i);
+       SET_MANAGER (manager, showonlyiconic, i);
+      }
       else if (!strcasecmp (option1, "font")) {
+	char *f;
 	p = read_next_cmd (READ_REST_OF_LINE);
-    trim(p);
+	trim(p);
 	if (!p) {
 	  ConsoleMessage ("Bad line: %s\n", current_line);
 	  continue;
 	}
-	ConsoleDebug (CONFIG, "font: %s\n", p);
+	CopyStringWithQuotes(&f, p);
+	ConsoleDebug (CONFIG, "font: %s\n", f);
 
 	SET_MANAGER (manager, fontname,
-		     copy_string (&globals.managers[id].fontname, p));
+		     copy_string (&globals.managers[id].fontname, f));
+	free(f);
       }
       else if (!strcasecmp (option1, "foreground")) {
 	p = read_next_cmd (READ_ARG);
@@ -1476,10 +1511,14 @@ void read_in_resources (char *file)
 	  r = SHOW_DESKTOP;
 	else if (!strcasecmp (p, "page"))
 	  r = SHOW_PAGE;
+	else if (!strcasecmp (p, "screen"))
+	  r = SHOW_SCREEN;
 	else if (!strcasecmp (p, "!desk"))
 	  r = NO_SHOW_DESKTOP;
 	else if (!strcasecmp (p, "!page"))
 	  r = NO_SHOW_PAGE;
+	else if (!strcasecmp (p, "!screen"))
+	  r = NO_SHOW_SCREEN;
 	else {
 	  ConsoleMessage ("Bad line: %s\n", current_line);
 	  ConsoleMessage ("What kind of resolution is this?\n");
@@ -1527,7 +1566,7 @@ void read_in_resources (char *file)
 	  ConsoleMessage ("What is this: %s?\n", p);
 	  continue;
 	}
-	if (i && globals.shapes_supported == 0) {
+	if (!FHaveShapeExtension && i) {
 	  ConsoleMessage ("Shape support not compiled in\n");
 	  continue;
 	}
