@@ -50,11 +50,17 @@
 #include <X11/Xlocale.h>
 #endif
 
+#include "libs/fvwmlib.h"
+#include "libs/FScreen.h"
+#include "libs/FShape.h"
 #include "libs/Module.h"
 #include "libs/Picture.h"
 #include "libs/Colorset.h"
+#include "libs/fvwmsignal.h"
 #include "FvwmIdent.h"
 
+static RETSIGTYPE TerminateHandler(int);
+
 static char *MyName;
 static fd_set_size_t fd_width;
 static int fd[2];
@@ -69,7 +75,6 @@ static XFontSet fontset;
 
 static int screen;
 static int x_fd;
-static int ScreenWidth, ScreenHeight;
 
 static char *yes = "Yes";
 static char *no = "No";
@@ -87,7 +92,7 @@ static Window app_win;
 
 static EventMask mw_events =
   ExposureMask | ButtonPressMask | KeyPressMask |
-  ButtonReleaseMask | KeyReleaseMask;
+  ButtonReleaseMask | KeyReleaseMask | StructureNotifyMask;
 
 static Atom wm_del_win;
 
@@ -136,8 +141,51 @@ int main(int argc, char **argv)
       exit(1);
     }
 
-  /* Dead pipe == dead fvwm */
-  signal (SIGPIPE, DeadPipe);
+#ifdef HAVE_SIGACTION
+  {
+    struct sigaction  sigact;
+
+    sigemptyset(&sigact.sa_mask);
+    sigaddset(&sigact.sa_mask, SIGPIPE);
+    sigaddset(&sigact.sa_mask, SIGTERM);
+    sigaddset(&sigact.sa_mask, SIGQUIT);
+    sigaddset(&sigact.sa_mask, SIGINT);
+    sigaddset(&sigact.sa_mask, SIGHUP);
+# ifdef SA_INTERRUPT
+    sigact.sa_flags = SA_INTERRUPT;
+# else
+    sigact.sa_flags = 0;
+# endif
+    sigact.sa_handler = TerminateHandler;
+
+    sigaction(SIGPIPE, &sigact, NULL);
+    sigaction(SIGTERM, &sigact, NULL);
+    sigaction(SIGQUIT, &sigact, NULL);
+    sigaction(SIGINT,  &sigact, NULL);
+    sigaction(SIGHUP,  &sigact, NULL);
+  }
+#else
+  /* We don't have sigaction(), so fall back to less robust methods.  */
+#ifdef USE_BSD_SIGNALS
+  fvwmSetSignalMask( sigmask(SIGPIPE) |
+                     sigmask(SIGTERM) |
+                     sigmask(SIGQUIT) |
+                     sigmask(SIGINT) |
+                     sigmask(SIGHUP) );
+#endif
+  signal(SIGPIPE, TerminateHandler);
+  signal(SIGTERM, TerminateHandler);
+  signal(SIGQUIT, TerminateHandler);
+  signal(SIGINT,  TerminateHandler);
+  signal(SIGHUP,  TerminateHandler);
+#ifdef HAVE_SIGINTERRUPT
+  siginterrupt(SIGPIPE, 1);
+  siginterrupt(SIGTERM, 1);
+  siginterrupt(SIGQUIT, 1);
+  siginterrupt(SIGINT, 1);
+  siginterrupt(SIGHUP, 1);
+#endif
+#endif
 
   fd[0] = atoi(argv[1]);
   fd[1] = atoi(argv[2]);
@@ -157,11 +205,10 @@ int main(int argc, char **argv)
   Root = RootWindow(dpy, screen);
 
   InitPictureCMap(dpy);
+  FScreenInit(dpy);
   /* prevent core dumps if fvwm doesn't provide any colorsets */
   AllocColorset(0);
-
-  ScreenHeight = DisplayHeight(dpy,screen);
-  ScreenWidth = DisplayWidth(dpy,screen);
+  FShapeInit(dpy);
 
   SetMessageMask(fd, M_CONFIGURE_WINDOW | M_WINDOW_NAME | M_ICON_NAME
                  | M_RES_CLASS | M_RES_NAME | M_END_WINDOWLIST | M_CONFIG_INFO
@@ -175,7 +222,7 @@ int main(int argc, char **argv)
   while(tline != (char *)0) {
     if(strlen(tline)>1) {
       if(strncasecmp(tline, CatString3(MyName,"Font",""),Clength+4)==0) {
-        CopyString(&font_string,&tline[Clength+4]);
+        CopyStringWithQuotes(&font_string,&tline[Clength+4]);
       }
       else if(strncasecmp(tline,CatString3(MyName,"Fore",""), Clength+4)==0) {
         CopyString(&ForeColor,&tline[Clength+4]);
@@ -192,6 +239,11 @@ int main(int argc, char **argv)
       else if(strncasecmp(tline, "Colorset", 8) == 0){
         LoadColorset(&tline[8]);
       }
+      else if(strncasecmp(tline, XINERAMA_CONFIG_STRING,
+			  sizeof(XINERAMA_CONFIG_STRING) - 1) == 0){
+	FScreenConfigureModule(
+	  tline + sizeof(XINERAMA_CONFIG_STRING) - 1);
+      }
     }
     GetConfigLine(fd,&tline);
   }
@@ -273,12 +325,14 @@ void process_message(unsigned long type,unsigned long *body)
 
 /***********************************************************************
  *
- * Detected a broken pipe - time to exit
+ *  Procedure:
+ *	SIGPIPE handler - SIGPIPE means fvwm is dying
  *
- **********************************************************************/
-void DeadPipe(int nonsense)
+ ***********************************************************************/
+static RETSIGTYPE
+TerminateHandler(int sig)
 {
-  exit(0);
+  fvwmSetTerminate(sig);
 }
 
 /***********************************************************************
@@ -381,9 +435,6 @@ void list_end(void)
   unsigned long gcm;
   int lmax,height;
   XEvent Event;
-  Window JunkRoot, JunkChild;
-  int JunkX, JunkY;
-  unsigned int JunkMask;
   int x,y;
   XSetWindowAttributes attributes;
   int is_key_pressed = 0;
@@ -442,32 +493,51 @@ void list_end(void)
   mysizehints.min_width = mysizehints.width;
   mysizehints.max_height = mysizehints.height;
   mysizehints.max_width = mysizehints.width;
-  XQueryPointer( dpy, Root, &JunkRoot, &JunkChild,
-		&x, &y, &JunkX, &JunkY, &JunkMask);
   mysizehints.win_gravity = NorthWestGravity;
+  {
+    int sx;
+    int sy;
+    int sw;
+    int sh;
+    Window JunkW;
+    int JunkC;
+    unsigned int JunkM;
+    fscreen_scr_arg fscr;
 
-  if((y+height+100)>ScreenHeight)
+    if (!XQueryPointer(
+	  dpy, Root, &JunkW, &JunkW, &x, &y, &JunkC, &JunkC, &JunkM))
     {
-      y = ScreenHeight - height - 10;
-      mysizehints.win_gravity = SouthWestGravity;
+      x = 0;
+      y = 0;
     }
 
-  if((x+lmax+100)>ScreenWidth)
+    fscr.xypos.x = x;
+    fscr.xypos.y = y;
+    FScreenGetScrRect(&fscr, FSCREEN_XYPOS, &sx, &sy, &sw, &sh);
+    if (y + height + 100 > sy + sh)
     {
-      x = ScreenWidth - lmax - 10;
-      if((y+height+100)>ScreenHeight)
+      y = sy + sh - height - 10;
+      mysizehints.win_gravity = SouthWestGravity;
+    }
+    if (x + lmax + 100 > sx + sw)
+    {
+      x = sx + sw - lmax - 10;
+      if (mysizehints.win_gravity == SouthWestGravity)
 	mysizehints.win_gravity = SouthEastGravity;
       else
 	mysizehints.win_gravity = NorthEastGravity;
     }
+  }
   mysizehints.x = x;
   mysizehints.y = y;
 
-
-  if(Pdepth < 2) {
+  if (Pdepth < 2)
+  {
     attributes.background_pixel = GetColor("white");
     fore_pix = GetColor("black");
-  } else {
+  }
+  else
+  {
     attributes.background_pixel = (colorset < 0)
 				  ? GetColor(BackColor)
     				  : Colorset[colorset].bg;
@@ -505,10 +575,12 @@ void list_end(void)
 
   /* Window is created. Display it until the user clicks or deletes it. */
   /* also grok any dynamic config changes */
-  while(1) {
+  while(1)
+  {
     FvwmPacket* packet;
     int x_fd = XConnectionNumber(dpy);
     fd_set fdset;
+    char buf[32];
 
     FD_ZERO(&fdset);
     FD_SET(fd[1], &fdset);
@@ -539,7 +611,6 @@ void list_end(void)
 	    if (is_button_pressed == 2 && Event.xbutton.button == 2)
 	    {
 	      /* select a new window when button 2 is pressed */
-	      fvwmlib_get_target_window(dpy, screen, MyName, &app_win, True);
 	      SetMessageMask(
 		fd, M_CONFIGURE_WINDOW | M_WINDOW_NAME | M_ICON_NAME
 		| M_RES_CLASS | M_RES_NAME | M_END_WINDOWLIST | M_CONFIG_INFO
@@ -547,6 +618,7 @@ void list_end(void)
 	      SendInfo(fd,"Send_WindowList",0);
 	      XDestroyWindow(dpy, main_win);
 	      DestroyList();
+	      fvwmlib_get_target_window(dpy, screen, MyName, &app_win, True);
 	      found = 0;
 	      return;
 	    }
@@ -559,6 +631,12 @@ void list_end(void)
         case ClientMessage:
 	  if (Event.xclient.format==32 && Event.xclient.data.l[0]==wm_del_win)
 	    exit(0);
+	  break;
+	case ReparentNotify:
+	  sprintf(buf, "Layer 0 %d", (int)target.layer);
+	  SendInfo(fd, buf, main_win);
+	  SendInfo(fd, "Raise", main_win);
+	  break;
 	case ConfigureNotify:
 	{
 	  /* this only happens with transparent windows, slurp up as many
@@ -595,6 +673,8 @@ void list_end(void)
       char *tline, *token;
 
       packet = ReadFvwmPacket(fd[1]);
+      if (packet == NULL)
+	exit(0);
       if (colorset >= 0 && packet && packet->type == M_CONFIG_INFO) {
 	tline = (char*)&(packet->body[3]);
 	tline = GetNextToken(tline, &token);
@@ -606,7 +686,7 @@ void list_end(void)
 				mysizehints.height,
 				&Colorset[colorset], Pdepth, gc,
 				True);
-	    /* ask for movement events iff transparent */
+	    /* ask for movement events if transparent */
 	    if (Colorset[colorset].pixmap == ParentRelative)
 	      mw_events |= StructureNotifyMask;
 	    else
@@ -614,6 +694,9 @@ void list_end(void)
 	    XSelectInput(dpy, main_win, mw_events);
 	  }
 	}
+	else if (StrEquals(token, XINERAMA_CONFIG_STRING)) {
+	  FScreenConfigureModule(tline);
+	}
 	free(token);
       }
     }
@@ -810,13 +893,13 @@ void MakeList(void)
   x1 = target.frame_x;
   if(x1 < 0)
     x1 = 0;
-  x2 = ScreenWidth - x1 - target.frame_w;
+  x2 = DisplayWidth(dpy,screen) - x1 - target.frame_w;
   if(x2 < 0)
     x2 = 0;
   y1 = target.frame_y;
   if(y1 < 0)
     y1 = 0;
-  y2 = ScreenHeight - y1 -  target.frame_h;
+  y2 = DisplayHeight(dpy,screen) - y1 -  target.frame_h;
     if(y2 < 0)
     y2 = 0;
   width = (width - target.base_w)/target.width_inc;
