@@ -94,8 +94,8 @@ void cool_get_inboxstatus();
 #define gray_height 8
 extern unsigned char gray_bits[];
 
-/*                x  y  w  h  tw th open type *text   win */
-TipStruct Tip = { 0, 0, 0, 0,  0, 0,   0,   0, NULL, None };
+/*                x  y  w  h  tw th open type px py *text   win */
+TipStruct Tip = { 0, 0, 0, 0,  0, 0,   0,   0, 0, 0, NULL, None };
 
 
 static void CreateOrUpdateGoodyGC(void)
@@ -145,11 +145,35 @@ static void CreateOrUpdateGoodyGC(void)
 
 Bool change_goody_colorset(int cset, Bool force)
 {
-  if (cset < 0 || (cset != tipscolorset && !force))
-    return False;
-  CreateOrUpdateGoodyGC();
+	if (cset < 0)
+	{
+		return False;
+	}
+	if (cset == tipscolorset && Tip.win != None)
+	{
+		char *s;
 
-  return True;
+		if (Tip.text != NULL)
+		{
+			s = safestrdup(Tip.text);
+		}
+		else
+		{
+			s = NULL;
+		}
+		PopupTipWindow(Tip.px, Tip.py, s);
+		if (s != NULL)
+		{
+			free(s);
+		}
+	}
+	if (!force && cset != colorset)
+	{
+		return False;
+	}
+	CreateOrUpdateGoodyGC();
+
+	return True;
 }
 
 static char *goodyopts[] =
@@ -417,6 +441,8 @@ void PopupTipWindow(int px, int py, const char *text)
   int newx, newy;
   Window child;
 
+  Tip.px = px;
+  Tip.py = py;
   if (!ShowTips)
     return;
   if (Tip.win != None)
